searchData={"items":[{"type":"module","title":"Tundra","doc":"TUN device support for Elixir.\n\nTundra provides a simple API for creating and using TUN devices on Linux and Darwin.\n\nAs TUN device creation is a privileged operation on most systems, Tundra uses a\nserver process to create and configure TUN devices. Once created, the device is\nrepresented within the runtime as a socket on Darwin and a NIF resource on Linux,\nwith process-ownership semantics i.e.\n\n- Only the owning process can read from or write to the device and receive i/o\n  notifications from it.\n- The TUN device is removed when the owning process exits.","ref":"Tundra.html"},{"type":"module","title":"Server Process - Tundra","doc":"The server program is called `tundra_svr` and is located under the `priv` directory\nof the Tundra application. The `:tundra` application expects that this server is\nalready running as a privileged user when it starts.\n\nThe server listens on a Unix domain socket at `/var/run/tundra.sock` and accepts\nrequests from the NIF to create and configure the tun device. The resulting\nfile descriptor is sent back to the NIF, which then creates a socket from it (on\nDarwin) or wraps it in a NIF resource (on Linux).","ref":"Tundra.html#module-server-process"},{"type":"module","title":"Non-blocking I/O - Tundra","doc":"Tundra only supports non-blocking I/O on TUN devices. The `recv/3` and `send/3`\nfunctions currently require that `:nowait` is pssed as the last argument. If the\noperation would block, the function will return `{:select, select_info}` and\na notification of the following form will be sent to the owning process when\nthe device is ready:\n\n    {:\"$socket\", dev, :select, select_handle}\n\nNote that, although on Linux the underlying TUN device is not technically a\nsocket, the same notification is used to reduce platform specific code. Only\nthe contents of `dev` differ.","ref":"Tundra.html#module-non-blocking-i-o"},{"type":"module","title":"IPv6 - Tundra","doc":"Tundra is designed to work with IPv6 and has only been tested with IPv6.","ref":"Tundra.html#module-ipv6"},{"type":"module","title":"Example - Tundra","doc":"The following GenServer creates a TUN device and then relays packets by\n\n1. Reading a frame from the device\n2. Swapping the source and destination addresses\n3. Writing the modified frame back to the device\n\n```elixir\ndefmodule Reflector do\n  use GenServer\n\n  @mtu 1500\n\n  def start_link(_), do: GenServer.start_link(__MODULE__, [], name: __MODULE__)\n\n  @impl true\n  def init(_args) do\n    # Create a TUN device with the given address and options\n    {:ok, state} =\n      Tundra.create(\"fd11:b7b7:4360::2\",\n        dstaddr: \"fd11:b7b7:4360::1\",\n        netmask: \"ffff:ffff:ffff:ffff::\",\n        mtu: @mtu)\n    {:ok, state, {:continue, :read}}\n  end\n\n  @impl true\n  def handle_continue(:read, {dev, _} = state) do\n    # Read a frame from the device\n    case Tundra.recv(dev, @mtu, :nowait) do\n      {:ok, data} ->\n        {:noreply, state, {:continue, {:reflect, data}}}\n      {:select, _} ->\n        {:noreply, state}\n    end\n  end\n\n  def handle_continue({:reflect, data}, {dev, _} = state) do\n    # Swap the source and destination addresses of the ipv6 frame\n    < > = data\n    reflected = [hdr, pre, dst, src, rest]\n\n    # Write the frame back to the device, assume it won't block\n    :ok = Tundra.send(dev, reflected, :nowait)\n    {:noreply, state, {:continue, :read}}\n  end\n\n  @impl true\n  def handle_info({:\"$socket\", dev, :select, _}, {dev, _} = state) do\n    # Handle 'input ready' notifications\n    {:noreply, state, {:continue, :read}}\n  end\n\nend\n```","ref":"Tundra.html#module-example"},{"type":"function","title":"Tundra.cancel/2","doc":"Cancel a pending operation on a TUN device.","ref":"Tundra.html#cancel/2"},{"type":"function","title":"Tundra.close/1","doc":"Close a TUN device.","ref":"Tundra.html#close/1"},{"type":"function","title":"Tundra.controlling_process/2","doc":"Transfer control of a TUN device to another process.\n\nMust be called by the current owner of the device.","ref":"Tundra.html#controlling_process/2"},{"type":"function","title":"Tundra.create/2","doc":"Create a TUN device.\n\nCreates a new TUN device with the given address and options. The options currently\nsupported are:\n\n- `:netmask` - The netmask of the device.\n- `:dstaddr` - The destination address of the device.\n- `:mtu` - The maximum transmission unit of the device.\n\nOn success returns a tuple containing a device tuple and the name of the device.","ref":"Tundra.html#create/2"},{"type":"function","title":"Examples - Tundra.create/2","doc":"iex> Tundra.create(\"fd11:b7b7:4360::2\",\n            dstaddr: \"fd11:b7b7:4360::1\",\n            netmask: \"ffff:ffff:ffff:ffff::\",\n            mtu: 16000)\n    {:ok, {{:\"$socket\", #Reference<0.2990923237.3512074243.109526>}, \"utun6\"}} # Darwin\n    {:ok, {{:\"$tundra\", #Reference<0.2990923237.3512074243.109526>}, \"tun0\"}}  # Linux","ref":"Tundra.html#create/2-examples"},{"type":"function","title":"Tundra.recv/3","doc":"Receive data from a TUN device.\n\nThe `length` argument specifies the maximum number of bytes to read. The caller\nis responsible for ensuring this length is at least as large at the MTU of the\ndevice, plus the size of the TUN header (4 bytes).\n\nThe `:nowait` option specifies that the operation should not block if no data is\navailable. If data is available, it will be returned immediately. If no data is\navailable, the function will return `{:select, select_info}`.","ref":"Tundra.html#recv/3"},{"type":"function","title":"Tundra.send/3","doc":"Send data to a TUN device.\n\nThe `data` argument is an iodata that will be written to the device. The data should\ninclude the 4-byte header that is expected by the TUN device.\n\nThe `:nowait` option specifies that the operation should not block if the device's\noutput buffer is full. If the buffer is full, the function will return\n`{:select, select_info}`.","ref":"Tundra.html#send/3"},{"type":"type","title":"Tundra.tun_address/0","doc":"A TUN device address. May be represented either as tuple or a\nstring containing a dotted IP address.","ref":"Tundra.html#t:tun_address/0"},{"type":"type","title":"Tundra.tun_device/0","doc":"A TUN device.\n\nOn Darwin, this is a socket. On Linux, this is a reference to a NIF resource.","ref":"Tundra.html#t:tun_device/0"},{"type":"type","title":"Tundra.tun_option/0","doc":"A TUN device creation option.","ref":"Tundra.html#t:tun_option/0"},{"type":"extras","title":"MIT License","doc":"# MIT License\n\nCopyright (c) 2025 Nick Gunn\n\nPermission is hereby granted, free of charge, to any person obtaining\na copy of this software and associated documentation files (the\n\"Software\"), to deal in the Software without restriction, including\nwithout limitation the rights to use, copy, modify, merge, publish,\ndistribute, sublicense, and/or sell copies of the Software, and to\npermit persons to whom the Software is furnished to do so, subject to\nthe following conditions:\n\nThe above copyright notice and this permission notice shall be\nincluded in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,\nEXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF\nMERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND\nNONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE\nLIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION\nWITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"},{"type":"extras","title":"CHANGELOG","doc":"# CHANGELOG","ref":"changelog.html"},{"type":"extras","title":"0.1.9 - 2025-01-12 - CHANGELOG","doc":"- Internal CI/CD changes only","ref":"changelog.html#0-1-9-2025-01-12"},{"type":"extras","title":"0.1.3 - 2025-01-12 - CHANGELOG","doc":"- Typos\n- mix format","ref":"changelog.html#0-1-3-2025-01-12"},{"type":"extras","title":"0.1.2 - 2025-01-12 - CHANGELOG","doc":"- Fix up some hex package stuff","ref":"changelog.html#0-1-2-2025-01-12"},{"type":"extras","title":"0.1.1 - 2025-01-12 - CHANGELOG","doc":"- Fix source_ref in docs generation","ref":"changelog.html#0-1-1-2025-01-12"},{"type":"extras","title":"0.1.0 - 2025-01-12 - CHANGELOG","doc":"Initial revision","ref":"changelog.html#0-1-0-2025-01-12"},{"type":"extras","title":"Tundra","doc":"# Tundra\n\n[![Hex](https://img.shields.io/hexpm/v/tundra.svg)](https://hex.pm/packages/tundra)\n[![Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/tundra/)\n[![License](https://img.shields.io/hexpm/l/tundra.svg)](https://github.com/ausimian/tundra/blob/master/LICENSE.md)\n![github actions badge](https://github.com/ausimian/tundra/actions/workflows/build.yaml/badge.svg)\n\nTUN device support for Elixir.\n\nTundra provides a simple API for creating and using TUN devices on Linux and Darwin.\n\nAs TUN device creation is a privileged operation on most systems, Tundra uses a\nserver process to create and configure TUN devices. Once created, the device is\nrepresented within the runtime as a socket on Darwin and a NIF resource on Linux,\nwith process-ownership semantics i.e.\n\n- Only the owning process can read from or write to the device and receive i/o\n  notifications from it.\n- The TUN device is removed when the owning process exits.","ref":"readme.html"},{"type":"extras","title":"Installation - Tundra","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `tundra` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:tundra, \"~> 0.1\"}\n  ]\nend\n```","ref":"readme.html#installation"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}